@@ -1,7 +1,8 @@
 use async_trait::async_trait;
 use serde::de::DeserializeOwned;
 use serde::Serialize;
-use sqlx::{Sqlite, Transaction};
+use sqlx::pool::PoolConnection;
+use sqlx::Sqlite;
 use uuid::Uuid;
 
 use crate::esrs::store::StoreEvent;
@@ -11,11 +12,7 @@ use crate::esrs::store::StoreEvent;
 pub trait SqliteProjector<Event: Serialize + DeserializeOwned + Clone + Send + Sync, Error> {
     /// This function projects one event in each read model that implements this trait.
     /// The result is meant to catch generic errors.
-    async fn project<'c>(
-        &self,
-        event: &StoreEvent<Event>,
-        transaction: &mut Transaction<'c, Sqlite>,
-    ) -> Result<(), Error>;
+    async fn project(&self, event: &StoreEvent<Event>, connection: &mut PoolConnection<Sqlite>) -> Result<(), Error>;
 }
 
 /// Projector trait that takes a Sqlite transaction in order to delete a read model
@@ -25,5 +22,5 @@ pub trait SqliteProjectorEraser<Event: Serialize + DeserializeOwned + Clone + Se
 {
     /// Delete the read model entry. It is here because of the eventual need of delete an entire
     /// aggregate.
-    async fn delete<'c>(&self, aggregate_id: Uuid, transaction: &mut Transaction<'c, Sqlite>) -> Result<(), Error>;
+    async fn delete(&self, aggregate_id: Uuid, transaction: &mut PoolConnection<Sqlite>) -> Result<(), Error>;
 }
